// C Source File
// Created 7/18/2011; 1:33:08 AM

#include <tigcclib.h>
/*******************************************************************************/
/* The Legend of Zelda: Link's Awakening                                       */
/* By: Jason Ho                                                                */
/*                                                                             */
/* This header contains data pertaining to objects in the game. Objects        */
/* include such things as hearts, crystals, bombs, arrows, swords, shields,    */
/* etc. Objects also include such animations like the sword swing, explosions, */
/* dissolving leaves, etc. Anything that is not a terrain or mob is probably   */
/* an object.                                                                  */
/*******************************************************************************/



#include "del_all.h"
#include "all.h"




/*---------------------------------------------------------------------------*/
/* Pictures                                                                  */
/*---------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/


/*===========================================================================*/
/* This function handles whatever an object needs to do: functionality,      */
/* animation, etc.                                                           */
/*===========================================================================*/
void AnimateObject(void)
{
	int list_num = 0;
	int count1 = 0;  //Used for temporary storage. NOT TO BE USED BETWEEN FRAMES!
	
	//-------------------------------------------------
	// Stops checking for objects if the number is -1
	//-------------------------------------------------
	while (object_list[list_num][OBJECT_NUMBER] >= 0)
	{
		switch (object_list[list_num][OBJECT_NUMBER])
		{
			//----------------------------------------------------------------------------
			// Crystals and Hearts essentially do the same thing, with minor differences
			//----------------------------------------------------------------------------
			case CRYSTAL1:
			case HEART1:
				count1 = 0;
				switch (object_list[list_num][OBJECT_FRAME])
				{
					//--------------------------------------------------------------
					// These frames show the Heart bouncing (when it first appears
					//--------------------------------------------------------------
					case 0:
						object_list[list_num][OBJECT_Y] -= 4;
						DrawImage(OBJECT, SHADOW1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
						break;
					case 2:
						object_list[list_num][OBJECT_Y] += 4;
						break;
					case 4:
						object_list[list_num][OBJECT_Y] -= 2;
						DrawImage(OBJECT, SHADOW1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
						break;
					case 6:
						object_list[list_num][OBJECT_Y] += 2;
						break;
					case 8:
						object_list[list_num][OBJECT_Y] -= 1;
						DrawImage(OBJECT, SHADOW1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
						break;
					case 10:
						object_list[list_num][OBJECT_Y] += 1;
						break;
					
					//----------------------------------------------------------------------------
					// After a certain amount of time, the heart starts flashing, and disappears.
					// count1 stores when to draw picture or not (to create flashing)
					//----------------------------------------------------------------------------
					case 120 ... 145:
						if (object_list[list_num][OBJECT_FRAME] % 2)
							count1 = 1;
						else
							count1 = 0;
						break;
				}
				
				//----------------------------------------------
				// Draws the picture (either heart or crystal)
				//----------------------------------------------
				if (!count1)
				{
					switch (object_list[list_num][OBJECT_NUMBER])
					{
						case HEART1:
							DrawImage(OBJECT, HEART1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
							break;
						case CRYSTAL1:
							DrawImage(OBJECT, CRYSTAL1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
							break;
					}
				}
				
				//----------------------------------------------------------------
				// Checks if player gets a heart or crystal (the frame > 6 is so 
				// the player doesnt get the heart/crystal before he can see it
				//----------------------------------------------------------------
				if (object_list[list_num][OBJECT_FRAME] > 6)
				{
					switch (object_list[list_num][OBJECT_NUMBER])
					{
						case HEART1:
							if (CheckAreaCollision(link_x, link_y, 16, 16, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]+5,6,6))
								 DeleteObject(list_num);
							break;
						case CRYSTAL1:
							if (CheckAreaCollision(link_x, link_y, 16, 16, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]+1,6,13))
								 DeleteObject(list_num);
							break;
					}
				}
				
				//-------------------------------------------------------
				// Disappears if the player didnt get the heart in time
				//-------------------------------------------------------
				if (object_list[list_num][OBJECT_FRAME] == 146)
					DeleteObject(list_num);
				
				//------------------------------------
				// Increase the objects frame number
				//------------------------------------
				object_list[list_num][OBJECT_FRAME]++;
				break;
			
			
			//-----------------------------
			// Displayed when a mob dies.
			//-----------------------------
			case EXPLODE1:
				switch (object_list[list_num][OBJECT_FRAME])
				{
					case 0 ... 1:
						DrawImage(OBJECT, EXPLODE1, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]+4, A_REVERSE);
						DrawImage(OBJECT, EXPLODE1, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]-4, A_REVERSE);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 2 ... 3:
						DrawImage(OBJECT, EXPLODE1, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]+4, A_NORMAL);
						DrawImage(OBJECT, EXPLODE1, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]-4, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 4 ... 5:
						DrawImage(OBJECT, EXPLODE2, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]+4, A_REVERSE);
						DrawImage(OBJECT, EXPLODE2, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]-4, A_REVERSE);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 6 ... 7:
						DrawImage(OBJECT, EXPLODE2, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]+4, A_NORMAL);
						DrawImage(OBJECT, EXPLODE2, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]-4, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 8 ... 9:
						DrawImage(OBJECT, EXPLODE2, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 10 ... 11:
						DrawImage(OBJECT, EXPLODE3, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y], A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					default:
						//-----------------------------------------------------------------
						// Random chance of leaving a heart or crystal when the mob dies.
						//-----------------------------------------------------------------
						if (!random(8))
						{
							switch (random(3))
							{
								case 0:
									AddObject(HEART1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y]);
									break;
								case 1 ... 2:
									AddObject(CRYSTAL1, object_list[list_num][OBJECT_X], object_list[list_num][OBJECT_Y]);
									break;
							}
						}
						
						//----------------------------------------
						// Delete the explosion when it is over.
						//----------------------------------------
						DeleteObject(list_num);
				}
				break;
				
			//-------------------------------------------
			// Animated when Link chops a bush or grass
			//-------------------------------------------
			case LEAF1:
				switch (object_list[list_num][OBJECT_FRAME])
				{
					case 0:
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]+0, A_NORMAL);
						DrawImage(OBJECT, LEAF1, object_list[list_num][OBJECT_X]+4, object_list[list_num][OBJECT_Y]+1, A_NORMAL);
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]+6, object_list[list_num][OBJECT_Y]+3, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+10, object_list[list_num][OBJECT_Y]-1, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 1:
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+1, object_list[list_num][OBJECT_Y]-6, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+5, object_list[list_num][OBJECT_Y]+0, A_NORMAL);
						DrawImage(OBJECT, LEAF1, object_list[list_num][OBJECT_X]+1, object_list[list_num][OBJECT_Y]+9, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+7, object_list[list_num][OBJECT_Y]+9, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 2:
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+1, object_list[list_num][OBJECT_Y]-6, A_NORMAL);
						DrawImage(OBJECT, LEAF1, object_list[list_num][OBJECT_X]-1, object_list[list_num][OBJECT_Y]-1, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+9, object_list[list_num][OBJECT_Y]+5, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+10, object_list[list_num][OBJECT_Y]+11, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 3:
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]+1, object_list[list_num][OBJECT_Y]-8, A_NORMAL);
						DrawImage(OBJECT, LEAF4, object_list[list_num][OBJECT_X]-4, object_list[list_num][OBJECT_Y]+8, A_NORMAL);
						DrawImage(OBJECT, LEAF2, object_list[list_num][OBJECT_X]+11, object_list[list_num][OBJECT_Y]+4, A_NORMAL);
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]+12, object_list[list_num][OBJECT_Y]+11, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					case 4:
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]-1, object_list[list_num][OBJECT_Y]-10, A_NORMAL);
						DrawImage(OBJECT, LEAF4, object_list[list_num][OBJECT_X]-6, object_list[list_num][OBJECT_Y]+6, A_NORMAL);
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]+13, object_list[list_num][OBJECT_Y]+3, A_NORMAL);
						DrawImage(OBJECT, LEAF3, object_list[list_num][OBJECT_X]+12, object_list[list_num][OBJECT_Y]+12, A_NORMAL);
						object_list[list_num][OBJECT_FRAME]++;
						break;
					default:
						DeleteObject(list_num);
				}
				break;
		}
		
		//--------------------------
		// Moves on to next object
		//--------------------------
		list_num++;
	}
}


//-------------------------------------------------------------------
// Adds an object to object_list. Once it has found a spot
// for the object, it returns the number of the object in the list.
//-------------------------------------------------------------------
short AddObject(short object_number, short object_x, short object_y)
{
	short list_num = 0;
	
	while (object_list[list_num][OBJECT_NUMBER] >= 0)
		list_num++;
	
	object_list[list_num][OBJECT_NUMBER] = object_number;
	object_list[list_num][OBJECT_X] = object_x;
	object_list[list_num][OBJECT_Y] = object_y;
	object_list[list_num][OBJECT_FRAME] = 0;
	
	return list_num;
}


//-------------------------------------------------------------------
// Deletes an object from object_list by shifting all objects above
// it down. This way adding and deleting objects will never leave
// "holes" in object_list.
//-------------------------------------------------------------------
void DeleteObject(short list_num)
{
	int count1;
	
	while (list_num+1 < 50)
	{
		//Shifts all objects down.
		for (count1=0;count1<4;count1++)
			object_list[list_num][count1] = object_list[list_num+1][count1];
		
		list_num++;
	}
	//Makes the last object in the list -1
	object_list[list_num][OBJECT_NUMBER] = -1;
}






























