// C Source File
// Created 7/18/2011; 1:38:56 AM

#include <tigcclib.h>
/*---------------------------------------------------------------------------*/
/* Pictures                                                                  */
/*---------------------------------------------------------------------------*/
#include "del_all.h"
#include "all.h"


/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/

/*============================================================================*/
/* AnimateMob is used to define mob-specific behavior. It passes a mobs data  */
/* to MoveMob, which defines general movement patterns, to create a mobs      */
/* specific behavior.                                                         */
/*============================================================================*/

//---------------------------------------------------------------------------------
// MOB_NUMBER refers to the list number in mob_list
//
// MOB_X and MOB_Y keep track of where the mob is.
//
// MOB_FRAME applys to which frame a mobs animation is on. Each game loop this
// frame is increased. Bascially a mob program just defines what a mob does on each
// frame. This method is similar to how Link and Terrain animations are done too.
//
// MOB_DIRECTION stores what direction a mob is currently facing. 
//
// MOB_PAUSE is used to store how long a mob's movement
// is delayed. NOTE however that this IS NOT automatically done for you in AnimateMob
// because of so many variations on mob pauses, so you'll have to write your own 
// routine to stop the mob from moving. (basically MOB_PAUSE could be used as a 
// free variable like the MOB_VARs, but i just added it cause it was so common.
//
// MOB_VAR1 and MOB_VAR2 are used to store mob-specific data that doesnt 
// apply to every mob. Bascially, they are free variables for whatever you want 
// to use them for.
//
//---------------------------------------------------------------------------------

void AnimateMob(void)
{
	int list_num;
	int count1=0;	//Used as temporary variable. NOT TO BE USED BETWEEN FRAMES!
	
	//------------------------------------------------------
	// Cycles through each mob in mob_list
	//------------------------------------------------------
	for (list_num=0;list_num<5;list_num++)
	{		
		//---------------------------------------------------------------------
		// This routine applys to all mobs. When MOB_PAUSE is less than 0, 
		// they are reeling back from a blow (caused by a Sword, arrow, etc.)
		//---------------------------------------------------------------------
		if (mob_list[list_num][MOB_PAUSE] < 0)
		{
			mob_list[list_num][MOB_PAUSE]++;
			
			switch (link_direction)
			{
				case UP:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_Y] -= 10;
					break;
				case RIGHT:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] += 10;
					break;
				case DOWN:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_Y] += 10;
					break;
				case LEFT:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] -= 10;
					break;
			}
			//---------------------------------------------------------------------
			// Once mob recovers from the blow, make him explode if he dies.
			//---------------------------------------------------------------------
			if (mob_list[list_num][MOB_PAUSE] == 0)
			{
				AddObject(EXPLODE1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]);
				mob_list[list_num][MOB_NUMBER] = -1;
			}
		}
		
		switch(mob_list[list_num][MOB_NUMBER])
		{
			case CRAB1:	//Crabs are usually found on the beach
				//---------------------------------
				// Shows the crab shuffling about
				//---------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 3:
						DrawImage(MOB, CRAB1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 4 ... 6:
						DrawImage(MOB, CRAB2, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, CRAB1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				
				//-------------------------------------------------------
				// Occasionally the Crab will move with a speed of 3
				// MOB_VAR keeps track of how long it's moving at 3
				//-------------------------------------------------------
				if (mob_list[list_num][MOB_VAR1] == 0 && !random(50))
						mob_list[list_num][MOB_VAR1] = 70;
				if (mob_list[list_num][MOB_VAR1] > 0)
					mob_list[list_num][MOB_VAR1]--;
					
				if (mob_list[list_num][MOB_VAR1])
					MoveMob(list_num, MOB_WANDER, 3);
				else
					MoveMob(list_num, MOB_WANDER, 2);
				break;
				
			case BUTTERFLY1:	//Butterflies are usually found in town
				//-------------------------
				// Flutters the butterfly
				//-------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 2:
						DrawImage(MOB, BUTTERFLY1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 3 ... 4:
						DrawImage(MOB, BUTTERFLY2, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, BUTTERFLY1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				
				MoveMob(list_num, MOB_FLUTTER, 1);
				break;
				
			case MONSTER1:	//Classic Zelda Octapus-looking monster
				
				//------------------------------------------------------------------
				// count1 temporarily store direction offsets, which point DrawImage
				// to the right picture to draw. I know I could plug this into a 
				// formula, but doing it this way makes it much clearer.
				//------------------------------------------------------------------
				count1=0;
				switch(mob_list[list_num][MOB_DIRECTION])
				{
					case LEFT:
						count1 += 2;
					case DOWN:
						count1 += 2;
					case RIGHT:
						count1 += 2;
					case UP:
						break;
				}
				
				//-----------------------------------------
				// The monster occasionally stops moving.
				//-----------------------------------------
				if (mob_list[list_num][MOB_PAUSE] == 0 && !random(50))
						mob_list[list_num][MOB_PAUSE] = 10;
				if (mob_list[list_num][MOB_PAUSE] > 0)
				{
					mob_list[list_num][MOB_PAUSE]--;
					DrawImage(MOB, MONSTER1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
					break;
				}
				
				//-----------------------------------------------------------------------------
				// Draws mob, taking into account the direction its facing, and its shuffling
				//-----------------------------------------------------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 2:
						DrawImage(MOB, MONSTER1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 3 ... 4:
						DrawImage(MOB, MONSTER1+count1+1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, MONSTER1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				MoveMob(list_num, MOB_WANDER, 2);
				break;
				
			case URCHIN1:	//Spikey creatures from the beach
				//------------------------------------------
				// Just shows it moving. Thats all it does
				//------------------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 5:
						DrawImage(MOB, URCHIN1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 6 ... 10:
						DrawImage(MOB, URCHIN2, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 11 ... 15:
						DrawImage(MOB, URCHIN3, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 16 ... 20:
						DrawImage(MOB, URCHIN4, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, URCHIN1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				break;
				
			case DOG1:	//Dogs are usually found in town
				//------------------------------------------------------------------
				// count1 temporarily store direction offsets, which point DrawMob 
				// to the right picture to draw.
				//------------------------------------------------------------------
				count1=0;
				switch(mob_list[list_num][MOB_DIRECTION])
				{
					case UP:
					case LEFT:
						count1 += 2;
					default:
						break;
				}
				
				//-----------------------------------------------------------------------------
				// Draws whichever direction he is facing. The mob_list[list_num][MOB_VAR1] 
				// stores how high the dog is jumping, and is generated by MOB_HOP from MoveMob().
				//-----------------------------------------------------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 2:
						DrawImage(MOB, DOG1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						break;
					case 3 ... 4:
						DrawImage(MOB, DOG2+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, DOG1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				
				//-------------------------------------------------------------------------------------
				// The dog is sometimes traveling in two directions. This routine makes it so
				// when the dog is facing left or right, it is still also traveling up and down,
				// and vice versa. MOB_VAR2 stores a number 0, 1 or 2, which is used to determine
				// whether the dog will go UP or DOWN, and is chosen randomly when he stops moving.
				// If the variable is 2, it wont travel in 2 directions.
				//
				// NOTE: If the dog is already moving UP or DOWN, it doesnt use MOB_VAR2; 
				// it just moves LEFT or RIGHT according to which direction he is facing.
				//-------------------------------------------------------------------------------------
				if ((mob_list[list_num][MOB_DIRECTION] == LEFT || mob_list[list_num][MOB_DIRECTION] == RIGHT) &&
				     mob_list[list_num][MOB_PAUSE] == 0 && mob_list[list_num][MOB_VAR2] != 2)
				{
					if (mob_list[list_num][MOB_VAR2] == UP)
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]-1, 16, 16))
							mob_list[list_num][MOB_Y] -= 1;
					}
					else
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]+1, 16, 16))
							mob_list[list_num][MOB_Y] += 1;
					}
				}
				
				if ((mob_list[list_num][MOB_DIRECTION] == UP || mob_list[list_num][MOB_DIRECTION] == DOWN) &&
				     mob_list[list_num][MOB_PAUSE] == 0 && mob_list[list_num][MOB_VAR2] != 2)
				{
					if (mob_list[list_num][MOB_DIRECTION] == UP)
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X]-1, mob_list[list_num][MOB_Y], 16, 16))
							mob_list[list_num][MOB_X] -= 1;
					}
					else
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X]+1, mob_list[list_num][MOB_Y], 16, 16))
							mob_list[list_num][MOB_X] += 1;
					}
				}
				
				//--------------------------------------------------------------
				// Makes the dog randomly stop moving.
				//--------------------------------------------------------------
				if (mob_list[list_num][MOB_PAUSE] == 0 && !random(50))
				{
						mob_list[list_num][MOB_PAUSE] = random(15)+10;
						mob_list[list_num][MOB_VAR2] = random(3);
				}
				
				//----------------------------------------------------
				// If the dog resumes movement, change its direction
				//----------------------------------------------------
				if (mob_list[list_num][MOB_PAUSE] > 0)
				{
					mob_list[list_num][MOB_PAUSE]--;
					if (mob_list[list_num][MOB_PAUSE] == 0)
						mob_list[list_num][MOB_DIRECTION] = random(4);
					break;
				}
				
				MoveMob(list_num, MOB_HOP, 1);
				break;
			
			case CHICKEN1:
				//------------------------------------------------------------------
				// The chicken basically has the same program as the dog, but with 
				// very minor tweaks, which graphically make alot of difference.
				//------------------------------------------------------------------
				
				//------------------------------------------------------------------
				// count1 temporarily store direction offsets, which point DrawMob 
				// to the right picture to draw.
				//------------------------------------------------------------------
				count1=0;
				switch(mob_list[list_num][MOB_DIRECTION])
				{
					case UP:
					case LEFT:
						count1 += 2;
					default:
						break;
				}
				
				//------------------------------------------------------------------------
				// Checks if the chicken resumes movement. This is different from the dog 
				// in that it doesnt give the chicken a chance to animate when it is paused.
				// It also doesnt give him a new random direction.
				//------------------------------------------------------------------------
				if (mob_list[list_num][MOB_PAUSE] > 0)
				{
					mob_list[list_num][MOB_PAUSE]--;
					DrawImage(MOB, CHICKEN1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
					break;
				}
				
				//-----------------------------------------------------------------------------
				// Draws whichever direction the chicken is facing. The mob_list[list_num][MOB_VAR1] 
				// stores how high the chicken is jumping, and is generated by MOB_HOP from MoveMob().
				//-----------------------------------------------------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 1 ... 4:	//Chicken's flapping wings doenst animate as fast as a dog walking
						DrawImage(MOB, CHICKEN1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						break;
					case 5 ... 8:
						DrawImage(MOB, CHICKEN2+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						break;
					default:	//This causes mob_frame reset when it has reached its max
						DrawImage(MOB, CHICKEN1+count1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y] - mob_list[list_num][MOB_VAR1], A_NORMAL);
						mob_list[list_num][MOB_FRAME] = 0;
				}
				
				//-------------------------------------------------------------------------------------
				// The chicken is sometimes traveling in two directions. This routine makes it so
				// when the chicken is facing left or right, it is still also traveling up and down,
				// and vice versa. MOB_VAR2 stores a number 0, 1 or 2, which is used to determine
				// whether the chicken will go UP or DOWN, and is chosen randomly when he stops moving.
				// If the variable is 2, it wont travel in 2 directions.
				//
				// NOTE: If the chicken is already moving UP or DOWN, it doesnt use MOB_VAR2; 
				// it just moves LEFT or RIGHT according to which direction he is facing.
				//-------------------------------------------------------------------------------------
				if ((mob_list[list_num][MOB_DIRECTION] == LEFT || mob_list[list_num][MOB_DIRECTION] == RIGHT) &&
				     mob_list[list_num][MOB_PAUSE] == 0 && mob_list[list_num][MOB_VAR2] != 2)
				{
					if (mob_list[list_num][MOB_VAR2] == UP)
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]-1, 16, 16))
							mob_list[list_num][MOB_Y] -= 1;
					}
					else
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]+1, 16, 16))
							mob_list[list_num][MOB_Y] += 1;
					}
				}
				
				if ((mob_list[list_num][MOB_DIRECTION] == UP || mob_list[list_num][MOB_DIRECTION] == DOWN) &&
				     mob_list[list_num][MOB_PAUSE] == 0 && mob_list[list_num][MOB_VAR2] != 2)
				{
					if (mob_list[list_num][MOB_DIRECTION] == UP)
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X]-1, mob_list[list_num][MOB_Y], 16, 16))
							mob_list[list_num][MOB_X] -= 1;
					}
					else
					{
						if (!CheckTerrainCollision(mob_list[list_num][MOB_X]+1, mob_list[list_num][MOB_Y], 16, 16))
							mob_list[list_num][MOB_X] += 1;
					}
				}
				
				//--------------------------------------------------------------
				// Makes the chicken randomly stop moving.
				// 
				// The chicken randomly stops alot more than the dog, but for 
				// shorter periods of time.
				//--------------------------------------------------------------
				if (mob_list[list_num][MOB_PAUSE] == 0 && !random(20))
				{
						mob_list[list_num][MOB_PAUSE] = 10;
						mob_list[list_num][MOB_VAR2] = 3;
				}
				
				MoveMob(list_num, MOB_HOP, 1);
				break;
				
			case SANDMOB1:	//Creatures that come up from the ground, twirl a bit, then digs itself back into the sand
			
				//------------------------------------------------------------
				// MOB_VAR1 stores how long hes in the ground. 0 if hes not.
				//------------------------------------------------------------
				if (mob_list[list_num][MOB_VAR1])
				{
					mob_list[list_num][MOB_VAR1]--;
					
					//-------------------------------------------------------------
					// Once the SandMob is ready to come back up, reset his frame
					//-------------------------------------------------------------
					if (mob_list[list_num][MOB_VAR1] == 0)
					{
						mob_list[list_num][MOB_FRAME] = 0;
						
						//---------------------------------------
						// Occasionally, rise from Link's feet.
						//---------------------------------------
						if(!random(3))
						{
							mob_list[list_num][MOB_X] = link_x;
							mob_list[list_num][MOB_Y] = link_y;
						}
					}
					break;
				}
					
				//-------------------------------------------------------------
				// Frames 0 - 15, and 80-90 show it rising from the sand.
				// Anything inbetween just shows the mob spinning. At frame
				// 91 the mob goes back into the ground, and the frames reset
				//-------------------------------------------------------------
				switch(mob_list[list_num][MOB_FRAME])
				{
					case 0 ... 5:
						DrawImage(MOB, SANDMOB1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 6 ... 10:
						DrawImage(MOB, SANDMOB2, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 11 ... 15:
						DrawImage(MOB, SANDMOB3, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 80 ... 85:
						DrawImage(MOB, SANDMOB2, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 86 ... 90:
						DrawImage(MOB, SANDMOB1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						break;
					case 91:
						mob_list[list_num][MOB_VAR1] = 40;
						break;
					default:	//If the mob is above the ground, twirl it, and move it.
						if (mob_list[list_num][MOB_FRAME] % 2)
							DrawImage(MOB, SANDMOB3, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						else
							DrawImage(MOB, SANDMOB4, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
						MoveMob(list_num, MOB_FOLLOW, 1);
				}
				break;
		}
	
	mob_list[list_num][MOB_FRAME]++;  //Move to the next frame.
	}
}


/*===========================================================================*/
/* MoveMob works with AnimateMob to define a mobs behavior. MoveMob defines  */
/* general movements patterns (such as MOB_WANDER and MOB_FLUTTER), which is */
/* modified by AnimateMob to create a mobs specific behavior.                */
/*                                                                           */
/* Takes in the list number from mob_list, the mobs general move type,       */
/* and the mobs movement speed.                                              */
/*===========================================================================*/

void MoveMob(int list_num, short move_type, short ani_speed)
{
	switch (move_type)
	{
		//-----------------------------------------------------------------------------
		// Constantly moves; if hit a wall, turn the opposite way - (example: CRAB1)
		//-----------------------------------------------------------------------------
		case MOB_WANDER:
			switch (mob_list[list_num][MOB_DIRECTION])
			{
				case RIGHT:
					//----------------------------------------------
					// If the mob wont run into anything, move him
					//----------------------------------------------
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X]+ani_speed, mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] += ani_speed;
					else
					//----------------------------------------------
					// Otherwise move him in the other direction
					//----------------------------------------------
						mob_list[list_num][MOB_DIRECTION] = LEFT;
					break;
				case LEFT:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X]-ani_speed, mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] -= ani_speed;
					else
						mob_list[list_num][MOB_DIRECTION] = RIGHT;
					break;
				case UP:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]-ani_speed, 16, 16))
						mob_list[list_num][MOB_Y] -= ani_speed;
					else
						mob_list[list_num][MOB_DIRECTION] = DOWN;
					break;
				case DOWN:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]+ani_speed, 16, 16))
						mob_list[list_num][MOB_Y] += ani_speed;
					else
						mob_list[list_num][MOB_DIRECTION] = UP;
					break;
			}
			
			//---------------------------------------------
			// Randomly change directions once in a while
			//---------------------------------------------
			if(!random(100))
			{
				mob_list[list_num][MOB_DIRECTION]--;
				if (mob_list[list_num][MOB_DIRECTION] < UP)
					mob_list[list_num][MOB_DIRECTION] = LEFT;
			}
			break;
		
		//---------------------------------------------------------------------
		// Move randomly - doesnt check for collision - (example: BUTTERFLY1)
		//---------------------------------------------------------------------
		case MOB_FLUTTER:
			switch (mob_list[list_num][MOB_DIRECTION])
			{
				//------------------------------------------------------------------------------------
				// These cases make the mob flutter towards a certain direction
				// 2-random(5) means the range of numbers is: -2 -1 0 1 2, meaning there is an equal opportunity to go any direction.
				// 2-random(2) means the range of numbers is: 1 2, meaning it moves toward a direction
				//------------------------------------------------------------------------------------
				case UP:
					mob_list[list_num][MOB_Y] -= 2-random(2);
					mob_list[list_num][MOB_X] += 2-random(5);
					break;
				case RIGHT:
					mob_list[list_num][MOB_Y] += 2-random(5);
					mob_list[list_num][MOB_X] += 2-random(2);
					break;
				case DOWN:
					mob_list[list_num][MOB_Y] += 2-random(2);
					mob_list[list_num][MOB_X] += 2-random(5);
					break;
				case LEFT:
					mob_list[list_num][MOB_Y] += 2-random(5);
					mob_list[list_num][MOB_X] -= 2-random(2);
					break;
			}
			//---------------------------------------------
			// Randomly change directions once in a while
			//---------------------------------------------
			if(!random(20))
			{
				mob_list[list_num][MOB_DIRECTION]--;
				if (mob_list[list_num][MOB_DIRECTION] < UP)
					mob_list[list_num][MOB_DIRECTION] = LEFT;
			}
			break;
		
		case MOB_HOP:
			//------------------------------------------------------------------------------
			// **NOTE**: To use this movement-type, MOB_VAR1 must be free. 
			//
			// MOB_HOP uses MOB_VAR1 to keep track of how many pixels a mob hops. To use
			// the hop, you actually have to add it to your mob defined in AnimateMob().
			//
			// This routine just constantly scrolls through MOB_VAR1, and draws a shadow
			// under the mob when it is high in the air (when MOB_VAR1 is 2)
			//------------------------------------------------------------------------------
			
			switch (mob_list[list_num][MOB_VAR1])
			{
				case 2:
					DrawImage(OBJECT, SHADOW1, mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y], A_NORMAL);
				case 0:
				case 1:
					mob_list[list_num][MOB_VAR1]++;
					break;
				default:
					mob_list[list_num][MOB_VAR1] = 0;
			}
			
			//--------------------------------------------------------------
			// Moves the mob (while hopping). It cannot 
			// move below pixel 2 since it is hopping 2 pixels each round.
			//--------------------------------------------------------------
			switch (mob_list[list_num][MOB_DIRECTION])
			{
				case RIGHT:
					//----------------------------------------------
					// If the mob wont run into anything, move him
					//----------------------------------------------
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X]+ani_speed, mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] += ani_speed;
					break;
				case LEFT:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X]-ani_speed, mob_list[list_num][MOB_Y], 16, 16))
						mob_list[list_num][MOB_X] -= ani_speed;
					break;
				case UP:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]-ani_speed-2, 16, 16))
						mob_list[list_num][MOB_Y] -= ani_speed;
					break;
				case DOWN:
					if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]+ani_speed, 16, 16))
						mob_list[list_num][MOB_Y] += ani_speed;
					break;
			}
			//---------------------------------------------
			// Randomly change directions once in a while
			//---------------------------------------------
			if(!random(40))
			{
				mob_list[list_num][MOB_DIRECTION]--;
				if (mob_list[list_num][MOB_DIRECTION] < UP)
					mob_list[list_num][MOB_DIRECTION] = LEFT;
			}
			break;
		
		//---------------------------------------------------
		// Follow makes the mob move towards where Link is.
		//---------------------------------------------------
		case MOB_FOLLOW:
		
		    //--------------------------------------------------------------------
			// These routines are pretty simple. Move the mob toward link, while 
			// checking if it hit anything in its path. 
			//--------------------------------------------------------------------
			if (mob_list[list_num][MOB_X] > link_x)
			{
				mob_list[list_num][MOB_DIRECTION] = LEFT;
				if (!CheckTerrainCollision(mob_list[list_num][MOB_X]-ani_speed, mob_list[list_num][MOB_Y], 16, 16))
					mob_list[list_num][MOB_X] -= ani_speed;
			}
			else
			{
				mob_list[list_num][MOB_DIRECTION] = RIGHT;
				if (!CheckTerrainCollision(mob_list[list_num][MOB_X]+ani_speed, mob_list[list_num][MOB_Y], 16, 16))
					mob_list[list_num][MOB_X] += ani_speed;
			}
				
			if (mob_list[list_num][MOB_Y] > link_y)
			{
				mob_list[list_num][MOB_DIRECTION] = UP;
				if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]-ani_speed-2, 16, 16))
					mob_list[list_num][MOB_Y] -= ani_speed;
			}
			else
			{
				mob_list[list_num][MOB_DIRECTION] = DOWN;
				if (!CheckTerrainCollision(mob_list[list_num][MOB_X], mob_list[list_num][MOB_Y]+ani_speed, 16, 16))
					mob_list[list_num][MOB_Y] += ani_speed;
			}
			
			break;
	}
	
	//---------------------------------------------------------
	// In all cases, the mob must not go out of screen bounds
	// If the mob reaches edge of screen, walk other way
	//---------------------------------------------------------
	if (mob_list[list_num][MOB_X] > 160-16)
	{
		mob_list[list_num][MOB_X] = 160-16;
		mob_list[list_num][MOB_DIRECTION] = LEFT;
	}
	if (mob_list[list_num][MOB_X] < 0)
	{
		mob_list[list_num][MOB_X] = 0;
		mob_list[list_num][MOB_DIRECTION] = RIGHT;
	}
	if (mob_list[list_num][MOB_Y] < 0)
	{
		mob_list[list_num][MOB_Y] = 0;
		mob_list[list_num][MOB_DIRECTION] = DOWN;
	}
	if (mob_list[list_num][MOB_Y] > 8*16-16)
	{
		mob_list[list_num][MOB_Y] = 8*16-16;
		mob_list[list_num][MOB_DIRECTION] = UP;
	}
}

//-------------------------------------------------------------------------
// SlashMob takes in an area, and determines if any mob lies in that area
//-------------------------------------------------------------------------
void SlashMob(short x, short y, short width, short height)
{
	int count1;
	
	for (count1=0;count1<5;count1++)
	{
		//------------------------------------------------------
		// Special mob cases
		//------------------------------------------------------
		switch (mob_list[count1][MOB_NUMBER])
		{
			//Butterfly is immune to hits
			case BUTTERFLY1:
				continue;
			//Sandmob is immune inside the ground.
			case SANDMOB1:
				if (mob_list[count1][MOB_VAR1] > 0)
					continue;
				break;
		}
		
		//------------------------------------------------------------------------------------------
		// If Link hits the mob, make mob reel back for 3 rounds. (this is explained in AnimateMob)
		//------------------------------------------------------------------------------------------
		if ( CheckAreaCollision(x,y,width,height, mob_list[count1][MOB_X], mob_list[count1][MOB_Y], 16, 16) &&
		     mob_list[count1][MOB_NUMBER] >= 0)
		{
			mob_list[count1][MOB_PAUSE] = -3;
		}
	}
}




