/****************************************************************************/
/* The Legend of Zelda: Link's Awakening                                    */
/* By: Jason Ho                                                             */
/*                                                                          */
/* This header contains data for terrain (defined in room_list of "room.h") */
/* Terrain includes things such as grass,rocks, trees, chests, etc.         */
/****************************************************************************/




//extern unsigned short int gfx_light[200][16],gfx_dark[200][16];
/*---------------------------------------------------------------------------*/
/* Aliases                                                                   */
/*---------------------------------------------------------------------------*/
//enum {TERRAIN_NUMBER, TERRAIN_X, TERRAIN_Y, TERRAIN_FRAME}; //Used in terrain_animation_list
// {TERRAIN_PIC, TERRAIN_PROPERTIES}; //References to terrain data (defined below)

//-------------------
// Bitwise booleans
//-------------------
/*#define WALKABLE    0x01
#define ANIMATED    0x02
#define ENTERABLE   0x04
#define FALLABLE    0x08
#define LIFTABLE    0x10
#define BOMBABLE    0x20
*/

// Header File
// Created 7/20/2011; 12:52:17 AM


#define GRASS1    0// slashable
#define HOUSE3    1
#define HOUSE4    2
#define STAIR1    3
#define SAND6     4

#define ROAD4     5
#define FLOOR1    6
#define FLOOR2    7
#define CABINET2  8
#define DOOR2     9

#define PHONE1    10
#define PHONE3    11
#define SHELF2    12

// expandable x3
#define EDGE13    16
#define EDGE18    17

// so up to 18 [17] is walkable.
//18->30 is walkable w/ 30 being the flower
#define ROAD1   18  // digableLEAF
#define ROAD2   19  // digable  question these roads for diggable
#define ROAD3   20  // digable
#define SAND1   21  // digable
#define SAND2   22  // digable
#define SAND3   23  // digable
#define SAND5   24  // digable
#define GRASS2  25  // diggable
// 2 more defines, possible 4 more that will be diggable.
// we will make it so that there are 12 possible diggable areas.

// expandable x4

#define FLOWER1 30  // digable
#define FLOWER2 31  
#define FLOWER3 32  
#define FLOWER4 33  
// 31 walk, 24 animated, 8 walk/animated, 78 other  ... 30 walk. 32 animated (8 objects).  
// 20 walking animated? (5 objects)
// 5 for holes that fall and cause you damage.
// 5 for clifs
// 82 used by that. (92)
// 0->29 walkable
// 30->49 walk/animated
// 50 -> 81 animated/non walkable
// 82 -> 174 all others
// 175 -> 184 liftable
// 185-> 194 hookshot
// 195 -> 199 hook/lift
// you also need diggable. blah  - desiginate 5 of the 30 walk as diggable.

#define SHORE1   34 
#define SHORE2   35 
#define SHORE3   36 
#define SHORE4   37 

// there exists a walkable animated water that looks like the river in afw.
// expandable x12

#define LIGHT1  50  
#define LIGHT2  51  
#define LIGHT3  52  
#define LIGHT4  53  


#define WATER1   54 
#define WATER2   55 
#define WATER3   56 
#define WATER4   57 

#define OBJECT1   58
#define OBJECT3   59
#define OBJECT4   60
#define OBJECT5   61

#define WATER5    62
#define WATER6    63
#define WATER7    64
#define WATER8    65

// expandable x16

#define PLANT1  82  // slashable
#define FENCE1  83  
#define TREE1   84  
#define TREE2   85  
#define TREE3   86  

#define TREE4   87  
#define ROOF10  88  
#define HOUSE2  89  
#define EDGE1   90  
#define EDGE2   91  

#define EDGE3   92  
#define EDGE4   93  
#define EDGE5   94  
#define HOUSE5   95 
#define TREE5    96 

#define ROOF1    97 
#define ROOF2    98 
#define ROOF3    99 
#define ROOF4    100 
#define ROOF5    101 

#define ROOF6    102 
#define FENCE2   103 
#define ROOF7    104 
#define ROOF8    105 
#define ROOF9    106 

#define TREE6    107 
#define FENCE3   108 
#define FENCE4   109 
#define EDGE6    110 
#define EDGE7    111 

#define EDGE8    112 
#define EDGE9    113 
#define CLIFF1   114 
#define CLIFF2   115 
#define CLIFF3    116

#define OBJECT2   117
#define EDGE10    118
#define EDGE11    119
#define EDGE12    120
#define EDGE14    121

#define EDGE15    122
#define SIGN1     123
#define EDGE16    124
#define CLIFF4    125
#define WELL1     126

#define EDGE17    127
#define EDGE19    128
#define DUNG1     129
#define DUNG2     130
#define DUNG3     131

#define DUNG4     132
#define PALM1     133
#define PALM2     134
#define PALM3     135
#define PALM4     136

#define SAND4     137
#define CLIFF5    138
#define CLIFF6    139
#define EDGE20    140
#define WALL1     141

#define WALL2     142
#define WALL3     143
#define WALL4     144
#define WALL5     145
#define WALL6     146

#define WALL7     147
#define WALL8     148
#define TABLE1    149
#define TABLE2    150
#define CHAIR1    151

#define BOX1      152
#define BED1      153
#define BED2      154
#define CABINET1  155
#define TABLE3    156

#define BLACK1    157
#define PHONE2    158
#define PHONE4    159
#define CASHIER1  160
#define FLOOR3    161

#define WHITE1    162
#define TABLE4    163
#define SHELF1    164

#define CHEST1    185// hookshot
#define POT1      195// lift
#define ROCK1     175// lift
#define HOLE1     170// hole

//--------------------------------------------------------------------------------------------------------------
// The following data is organized:
// #define NEW_TERRAIN {TERRAIN_PIC, TERRAIN_PROPERTIES}
// where:
// 1) TERRAIN_PIC is the picture number in img_map_light and img_map_dark, defined in "images.h"
// 2) TERRAIN_PROPERTIES is a byte, with each bit representing a boolean. These booleans are shown in the Alias section
//    of this header.
//
// To add another slot of data, you must also update: 1) TERRAIN value of room_list, 2) size of room_buffer, and
// 3) count3 in function LoadRoom()
//---------------------------------------------------------------------------------------------------------------

/*
#define GRASS1    {1,WALKABLE}// slashable
#define HOUSE3    {10,WALKABLE | ENTERABLE}
#define HOUSE4    {18,WALKABLE | ENTERABLE}
#define STAIR1    {43,WALKABLE}
#define SAND6     {119,WALKABLE}

#define ROAD4     {75,WALKABLE}
#define FLOOR1    {108,WALKABLE}
#define FLOOR2    {109,WALKABLE}
#define CABINET2  {115,WALKABLE}
#define DOOR2     {116,WALKABLE | ENTERABLE}

#define PHONE1    {120,WALKABLE}
#define PHONE3    {122,WALKABLE}
#define SHELF2    {133,WALKABLE}
#define EDGE13    {51,WALKABLE | FALLABLE}
#define EDGE18    {68,WALKABLE | FALLABLE}


// so up to 18 [17] is walkable.
//18->30 is walkable w/ 30 being the flower
#define ROAD1     {11,WALKABLE}// digable
#define ROAD2     {28,WALKABLE}// digable  question these roads for diggable
#define ROAD3     {29,WALKABLE}// digable
#define SAND1     {76,WALKABLE}// digable
#define SAND2     {81,WALKABLE}// digable
#define SAND3     {82,WALKABLE}// digable
#define SAND5     {93,WALKABLE}// digable
#define GRASS2    {2,WALKABLE}// diggable
// 2 more defines, possible 4 more that will be diggable.
// we will make it so that there are 12 possible diggable areas.
#define FLOWER1   {12,WALKABLE | ANIMATED}// digable
#define FLOWER2   {58,WALKABLE | ANIMATED}
#define FLOWER3   {59,WALKABLE | ANIMATED}
#define FLOWER4   {60,WALKABLE | ANIMATED}


#define SHORE1    {85,WALKABLE | ANIMATED}
#define SHORE2    {90,WALKABLE | ANIMATED}
#define SHORE3    {91,WALKABLE | ANIMATED}
#define SHORE4    {92,WALKABLE | ANIMATED}

// there exists a walkable animated water that looks like the river in afw.

#define LIGHT1    {124,ANIMATED}
#define LIGHT2    {125,ANIMATED}
#define LIGHT3    {126,ANIMATED}
#define LIGHT4    {127,ANIMATED}


#define WATER1    {20,ANIMATED}
#define WATER2    {61,ANIMATED}
#define WATER3    {62,ANIMATED}
#define WATER4    {63,ANIMATED}

#define OBJECT1   {45,ANIMATED}
#define OBJECT3   {64,ANIMATED}
#define OBJECT4   {65,ANIMATED}
#define OBJECT5   {66,ANIMATED}

#define WATER5    {86,ANIMATED}
#define WATER6    {87,ANIMATED}
#define WATER7    {88,ANIMATED}
#define WATER8    {89,ANIMATED}


#define PLANT1    {0,0}// slashable
#define FENCE1    {3,0}
#define TREE1     {4,0}
#define TREE2     {5,0}
#define TREE3     {6,0}
#define TREE4     {7,0}
#define ROOF10    {8,0}
#define HOUSE2    {9,0}

#define EDGE1     {13,0}
#define EDGE2     {14,0}
#define EDGE3     {15,0}
#define EDGE4     {16,0}
#define EDGE5     {17,0}

#define HOUSE5    {19,0}

#define TREE5     {21,0}
#define ROOF1     {22,0}
#define ROOF2     {23,0}
#define ROOF3     {24,0}
#define ROOF4     {25,0}
#define ROOF5     {26,0}
#define ROOF6     {27,0}

#define FENCE2    {30,0}
#define ROOF7     {31,0}
#define ROOF8     {32,0}
#define ROOF9     {33,0}
#define TREE6     {34,0}
#define FENCE3    {35,0}
#define FENCE4    {36,0}
#define EDGE6     {37,0}
#define EDGE7     {38,0}
#define EDGE8     {39,0}
#define EDGE9     {40,0}
#define CLIFF1    {41,0}
#define CLIFF2    {42,0}

#define CLIFF3    {44,0}

#define OBJECT2   {46,0}

#define EDGE10    {48,0}
#define EDGE11    {49,0}
#define EDGE12    {50,0}

#define EDGE14    {52,0}
#define EDGE15    {53,0}
#define SIGN1     {54,0}
#define EDGE16    {55,0}
#define CLIFF4    {56,0}
#define WELL1     {57,0}


#define EDGE17    {67,0}


#define EDGE19    {70,0}
#define DUNG1     {71,0}
#define DUNG2     {72,0}
#define DUNG3     {73,0}
#define DUNG4     {74,0}

#define PALM1     {77,0}
#define PALM2     {78,0}
#define PALM3     {79,0}
#define PALM4     {80,0}

#define SAND4     {83,0}




#define CLIFF5    {94,0}
#define CLIFF6    {95,0}
#define EDGE20    {96,0}
#define WALL1     {97,0}
#define WALL2     {98,0}
#define WALL3     {99,0}
#define WALL4     {100,0}
#define WALL5     {101,0}
#define WALL6     {102,0}
#define WALL7     {103,0}
#define WALL8     {104,0}
#define TABLE1    {105,0}
#define TABLE2    {106,0}


#define CHAIR1    {110,0}
#define BOX1      {111,0}
#define BED1      {112,0}
#define BED2      {113,0}
#define CABINET1  {114,0}

#define TABLE3    {117,0}
#define BLACK1    {118,0}

#define PHONE2    {121,0}

#define PHONE4    {123,0}

#define CASHIER1  {128,0}
#define FLOOR3    {129,0}
#define WHITE1    {130,0}
#define TABLE4    {131,0}
#define SHELF1    {132,0}

#define CHEST1    {84,0}// hookshot
#define POT1      {107,0}// lift
#define ROCK1     {47,0}// lift
#define HOLE1     {69,0}// hole

*/

// 31 walk, 24 animated, 8 walk/animated, 78 other  ... 30 walk. 32 animated (8 objects).  
// 20 walking animated? (5 objects)
// 5 for holes that fall and cause you damage.
// 5 for clifs
// 82 used by that. (92)
// 0->29 walkable
// 30->49 walk/animated
// 50 -> 81 animated/non walkable
// 82 -> 174 all others
// 175 -> 184 liftable
// 185-> 194 hookshot
// 195 -> 199 hook/lift
// you also need diggable. blah  - desiginate 5 of the 30 walk as diggable.

// desiginate some as slashable.

// 100 other.  you also need a set for hookshot grabable.  maybe from 175->199? 
// 2 fallable
// bombable will be handled by the sl
// need item type of boulder for the power braclet.
// need type water - only 2 places needed.
// 10 liftable objects
// 10 hookshot object
// 5 hook/lift



// object 1,3,4, 5 are the same item - animated - loop (object 2 bottom half)(birdie windmill)
// light has 4 to cycle through.
// water 1&2 has 4 to cycle through.
// flower has 4 bmps, but walkable
// tide has 4 to cycle though, but walkable  (interesting tide isnt labled)

extern short screen_shift;                  //screen shift is explained in function DrawTerrain() from "terrain.h"
extern short screen_row_gain;               //screen_row_gain is explained in function DrawTerrain() from "terrain.h"

extern unsigned short image_buffer_light[16];	 //Used to store temporarily modified pictures
extern unsigned short image_buffer_dark[16];    //Used to store temporarily modified pictures
extern unsigned short image_buffer_mask[16];    //Used to store temporarily modified pictures
/*---------------------------------------------------------------------------*/
/* Function Prototypes                                                       */
/*---------------------------------------------------------------------------*/
void DrawTerrain(void);
//void AnimateTerrain(void);

//short CheckTerrainCollision(short, short, short, short);
//void SlashTerrain(short, short);
//void EnterTerrain(short, short, short);

//short GetTerrainRow(short);
//short GetTerrainCol(short);


/*---------------------------------------------------------------------------*/
/* Pictures                                                                  */
/*---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------*/
/*  img_map_<light,dark> [TERRAIN_TYPE] [row]                                           */
/*  where TERRAIN_TYPE is any of TERRAINs defined in "terrain.h"                        */
/*        row is the pixel row of the picture                                           */
/*--------------------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/

/*===========================================================================*/
/* Draws Terrain of a room                                                   */
/*===========================================================================*/
// this might be a keeper.
/*void DrawTerrain(void)
{
	int count1,count2;

//------------------------------------------------------------------------------------------------
// Screen Shift Concept:
// Because the TI-89 screen can only fit 6 rows of tiles instead of 8 like the gameboy,
// A routine must be implemented to scroll the screen up and Down so that the player can see the whole room.
// screen_shift represents how any pixels from the top of the room the routine must scroll down.
// if screen_shift reaches 16, it resets and adds 1 to screen_row_gain
//
// Conceptually, the routine does:
// 1) Takes the first row pictures on the screen, and shifts the horizontal pixels up,
//    truncating its height to whatever it should be (using function ShiftImage).
// 2) Shifts the rest of the rows on the screen up to complete the picture.
//------------------------------------------------------------------------------------------------
	screen_shift = 0;
	screen_row_gain = 0;
	
	//--------------------------------------------------------------------
	// If Link goes beyond the 4th row of terrain, increase screen shift
	//--------------------------------------------------------------------
	if (link_y >= 3*16)
	{
		count1 = link_y - 3*16;
		screen_row_gain = (int)(count1/16);
		screen_shift = count1 % 16;
	}
	//--------------------------------------------------------------------------------------
	// Total screen shift must not exceed 28, because there are 16*2 rows, +4 extra pixels 
	// to display (because after 6 rows, 96 pixels, 4 pixels remain from LCD_HEIGHT
	//--------------------------------------------------------------------------------------
	if (screen_shift + screen_row_gain*16 > 2*16-4)
	{
		screen_row_gain = 1;
		screen_shift = 12;
	}
	
		
	for (count1=0;count1<10;count1++)	//Column
	{
		//----------------------------------------
		// Draws first row pictures with shift
		//----------------------------------------
		//ShiftImage(short pic_type, short pic_num, short direction, short shift, short masked)
		ShiftImage(TERRAIN, get_room_maps(rmapx(),rmapy(),screen_row_gain,count1), up, screen_shift, FALSE);
		Sprite16(count1*16, 0, 16, image_buffer_light, light, SPRT_XOR);
		Sprite16(count1*16, 0, 16, image_buffer_dark, dark, SPRT_XOR);
		
		//----------------------------------------------------------------
		// Starts on the 2nd row of pictures, and moves the pictures up.
		//----------------------------------------------------------------
		for (count2=1;count2<8;count2++)	//Row
		{
			Sprite16(count1*16,count2*16-screen_shift,16,gfx_light[ get_room_maps(rmapx(),rmapy(),count2+screen_row_gain,count1) ],light,SPRT_XOR);
			Sprite16(count1*16,count2*16-screen_shift,16,gfx_dark[ get_room_maps(rmapx(),rmapy(),count2+screen_row_gain,count1) ],dark,SPRT_XOR);
		}
	}
}*/


/*===========================================================================*/
/* Animates terrain on the screen by cycling through frames from             */
/* terrain_animation_list.                                                   */
/*===========================================================================*/
/*void AnimateTerrain(void)
{	
	int list_num = 0;	//Cycles through the terrain_animation_list
	
	//--------------------------------------------------
	// Defines how fast an animation moves. The larger 
	// ani_speed is, the slower it moves.
	//--------------------------------------------------
	int ani_speed = 0;
	
	//---------------------------------------------------
	// Each animated terrain cycles through 4 pictures.
	// ani_list holds the picture number for these pics
	//---------------------------------------------------
	int ani_list[] = {0,0,0,0};
	
	//-----------------------------------------------------
	// Scrolls through the animated terrains in the room
	//-----------------------------------------------------
	
	// hahahaha this can all be deleted.  i will rewrite the animation code myself.
	// it is all outdated now.  bwuhahah.  wow. this setup sucks.  it is pretty bad.
	while (terrain_animation_list[list_num][TERRAIN_NUMBER] >= 0)
	{
		switch (terrain_animation_list[list_num][TERRAIN_NUMBER])
		{
			case 12: //FLower1
				ani_speed = 7;
				ani_list[0] = 12;
				ani_list[1] = 58;
				ani_list[2] = 60;
				ani_list[3] = 59;
				break;			
			case 20: //Water1
				ani_speed = 8;
				ani_list[0] = 20;
				ani_list[1] = 61;
				ani_list[2] = 62;
				ani_list[3] = 63;
				break;
			case 45: //Flying Rooster
				ani_speed = 3;
				ani_list[0] = 45;
				ani_list[1] = 64;
				ani_list[2] = 65;
				ani_list[3] = 66;
				break;
			case 86: //Water5 (the water at the beach)
				ani_speed = 7;
				ani_list[0] = 86;
				ani_list[1] = 87;
				ani_list[2] = 88;
				ani_list[3] = 89;
				break;
			case 85: //Shore1 (shore water at the beach)
				ani_speed = 7;
				ani_list[0] = 85;
				ani_list[1] = 90;
				ani_list[2] = 91;
				ani_list[3] = 92;
				break;
			//-----------------------------------------------------
			// Light - found inside houses. This routine is a
			// little different, so it overrides the default
			// routine of handling terrain animation.
			//-----------------------------------------------------
			case 124:
				switch (terrain_animation_list[list_num][TERRAIN_FRAME])
				{
					case 1 ... 4*1:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 124;
						break;
					case 4*1+1 ... 4*2:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 125;
						break;
					case 4*2+1 ... 4*3:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 126;
						break;
					case 4*3+1 ... 4*4+10:	//Short delay when the light is fully on
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 127;
						break;
					case 4*4+1+10 ... 4*5+10:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 126;
						break;
					case 4*5+1+10 ... 4*6+10:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 125;
						break;
					default:
						terrain_animation_list[list_num][TERRAIN_FRAME] = 1;
					
				}
				terrain_animation_list[list_num][TERRAIN_FRAME]++;
				list_num++;
				continue;
		}
	
		//---------------------------------------------------------------------------
		//	Switches pictures, according to what frame the animated terrain is on
		//---------------------------------------------------------------------------		
		
		if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[0];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 2*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[1];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 2*ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 3*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[2];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 3*ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 4*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[3];
		}
		//--------------------------------------------------
		//	Restarts the frame cycle if it reaches an end.
		//--------------------------------------------------
		else
		{
			terrain_animation_list[list_num][TERRAIN_FRAME] = 1;
		}
	
		//-----------------------------------------------------------------------
		//	Increase the frame number and list_num once everything is processed
		//-----------------------------------------------------------------------
		terrain_animation_list[list_num][TERRAIN_FRAME]++;
		
		list_num++;
		
	}
}*/

// replace with location_checker()
/*==================================================================================*/
/* This function checks if the given rectangular area collides with a non-walkable  */
/* terrain. If it does, it returns TRUE. Otherwise, it returns false.               */
/* NOTE: this function assumes the maximum area given will be 16x16.                */
/*==================================================================================*/
/*short CheckTerrainCollision(short x, short y, short width, short height)
{	
	//-----------------------------------------------------------------------
	// Any 16x16 area can potentially collide with 4 different terrains. So
	// 4 if-checks must be done, one on each of the terrains. The different
	// number additions/subtractions are for more travel leeway.
	//-----------------------------------------------------------------------
	if ((!(room_buffer[GetTerrainRow(y)][GetTerrainCol(x)][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, GetTerrainCol(x)*16+6, GetTerrainRow(y)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)][GetTerrainCol(x)+1][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, (GetTerrainCol(x)+1)*16+6, GetTerrainRow(y)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)+1][GetTerrainCol(x)][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, GetTerrainCol(x)*16+6, (GetTerrainRow(y)+1)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)+1][GetTerrainCol(x)+1][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, (GetTerrainCol(x)+1)*16+6, (GetTerrainRow(y)+1)*16+3, 16-6-6-1, 16-9-3)))
		return TRUE;
	else
		return FALSE;
}*/


/*======================================================================*/
/* Given the row and column of the terrain to slash, handles            */
/* whatever that terrain's slash does.                                  */
/*======================================================================*/
/*void SlashTerrain(short row, short col)
{
	switch (room_buffer[row][col][TERRAIN_PIC])
	{
		//--------------------------------------------------------
		// If Link slashes PLANT1 or GRASS1, change it to GRASS2
		//--------------------------------------------------------
		case 0:	//PLANT1
		case 1: //GRASS1
			room_buffer[row][col][TERRAIN_PROPERTIES] = WALKABLE;
			room_buffer[row][col][TERRAIN_PIC] = 2;	//GRASS2
			
			//---------------------------------------------------
			// Add the dissolving leaf animation to object_list
			//---------------------------------------------------
			AddObject(LEAF1, col*16, row*16);
			
			//---------------------------------------------------
			// Random chance of getting a heart or crystal
			//---------------------------------------------------
			if (!random(8))
			{
				//-----------------------------------
				// More crystals appear than hearts
				//-----------------------------------
				switch (random(3))
				{
					case 0:
						AddObject(HEART1, col*16, row*16);
						break;
					case 1 ... 2:
						AddObject(CRYSTAL1, col*16, row*16);
						break;
				}
			}
			break;
	}
}

*/
/*======================================================================*/
/* Enters into an enterable terrain.                                    

		UMMM NO.  this will be handled by the warper.  baka.. sigh... 1k of space too...sigh.

/*======================================================================*/
/*void EnterTerrain(short current_room, short row, short col)
{
	unsigned int count1;
	
	for (count1=0;count1<15;count1++)
	{
		OSContrastDn();
		DelayGame(30000);
	}
	
	switch (current_room)
	{
		//To Place where link starts off entrances
		case 1:
			link_map_col = 0;
			link_map_row = 0;
			link_x = 4*16;
			link_y = 7*16-10;
			LoadRoom(38);
			break;
		case 38:
			link_map_col = 2;
			link_map_row = 10;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(1);
			break;
		//Bowwow house entrances
		case 5:
			if (col == 3)
			{
				link_map_col = 1;
				link_map_row = 0;
				link_x=4*16;
				link_y = 7*16-10;
				LoadRoom(39);
			}
			else
			{
				link_map_col = 2;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(40);
			}
			break;
		case 39:
			link_map_col = 1;
			link_map_row = 10;
			link_x = 3*16;
			link_y = 3*16+8;
			LoadRoom(5);
			break;
		case 40:
			link_map_col = 1;
			link_map_row = 10;
			link_x = 5*16;
			link_y = 3*16+8;
			LoadRoom(5);
			break;
		//Phone roof house entrances
		case 4:
			link_map_col = 3;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(41);
			break;
		case 41:
			link_map_col = 2;
			link_map_row = 11;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(4);
			break;
		//Ulrira house entrances
		case 6:
			link_map_col = 4;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(42);
			break;
		case 42:
			link_map_col = 1;
			link_map_row = 11;
			link_x = 4*16;
			link_y = 5*16+8;
			LoadRoom(6);
			break;
		//Papahl's house entrances
		case 12:
			if (col == 5)
			{
				link_map_col = 5;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(43);
			}
			else
			{
				link_map_col = 6;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(44);
			}
			break;
		case 43:
			link_map_col = 2;
			link_map_row = 8;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(12);
			break;
		case 44:
			link_map_col = 2;
			link_map_row = 8;
			link_x = 7*16;
			link_y = 4*16+8;
			LoadRoom(12);
			break;
		//Item shop entrances
		case 9:
			link_map_col = 7;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(45);
			break;
		case 45:
			link_map_col = 3;
			link_map_row = 9;
			link_x = 4*16;
			link_y = 5*16+8;
			LoadRoom(9);
			break;
		//Trendy game entrances
		case 3:
			link_map_col = 8;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(46);
			break;
		case 46:
			link_map_col = 3;
			link_map_row = 11;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(3);
			break;
		//School entrances
		case 17:
			link_map_col = 9;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(47);
			break;
		case 47:
			link_map_col = 0;
			link_map_row = 11;
			link_x = 3*16;
			link_y = 2*16+8;
			LoadRoom(17);
			break;
		//Alligator house entrances
		case 29:
			link_map_col = 10;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(48);
			break;
		case 48:
			link_map_col = 3;
			link_map_row = 14;
			link_x = 4*16;
			link_y = 2*16+8;
			LoadRoom(29);
			break;
	}
	
	DrawScreen();
	row++;
	
	for (count1=0;count1<9;count1++)
	{
		OSContrastUp();
		DelayGame(30000);
	}
}*/


/*=================================================*/
/* Calculates the terrain's row, given a y         */
/*=================================================*/
/*short GetTerrainRow(short y)
{
	return (int)(y/16);
}
*/

/*=================================================*/
/* Calculates the terrain's column, given an x     */
/*=================================================*/
/*short GetTerrainCol(short x)
{
	return (int)(x/16);
}
*/
















