// C Source File
// Created 7/18/2011; 1:37:17 AM

#include <tigcclib.h>

/*===========================================================================*/
/* Draws Terrain of a room                                                   */
/*===========================================================================*/
extern unsigned short int gfx_light[200][16],gfx_dark[200][16];
#include "del_all.h"
#include "all.h"
void DrawTerrain(void)
{
	int count1,count2;

//------------------------------------------------------------------------------------------------
// Screen Shift Concept:
// Because the TI-89 screen can only fit 6 rows of tiles instead of 8 like the gameboy,
// A routine must be implemented to scroll the screen Up and Down so that the player can see the whole room.
// screen_shift represents how any pixels from the top of the room the routine must scroll down.
// if screen_shift reaches 16, it resets and adds 1 to screen_row_gain
//
// Conceptually, the routine does:
// 1) Takes the first row pictures on the screen, and shifts the horizontal pixels up,
//    truncating its height to whatever it should be (using function ShiftImage).
// 2) Shifts the rest of the rows on the screen UP to complete the picture.
//------------------------------------------------------------------------------------------------
	screen_shift = 0;
	screen_row_gain = 0;
	
	//--------------------------------------------------------------------
	// If Link goes beyond the 4th row of terrain, increase screen shift
	//--------------------------------------------------------------------
	if (link_y >= 3*16)
	{
		count1 = link_y - 3*16;
		screen_row_gain = (int)(count1/16);
		screen_shift = count1 % 16;
	}
	//--------------------------------------------------------------------------------------
	// Total screen shift must not exceed 28, because there are 16*2 rows, +4 extra pixels 
	// to display (because after 6 rows, 96 pixels, 4 pixels remain from LCD_HEIGHT
	//--------------------------------------------------------------------------------------
	if (screen_shift + screen_row_gain*16 > 2*16-4)
	{
		screen_row_gain = 1;
		screen_shift = 12;
	}
	
		
	for (count1=0;count1<10;count1++)	//Column
	{
		//----------------------------------------
		// Draws first row pictures with shift
		//----------------------------------------
		ShiftImage(TERRAIN, room_buffer[screen_row_gain][count1][TERRAIN_PIC], UP, screen_shift, FALSE);
		Sprite16(count1*16, 0, 16, image_buffer_light, light, SPRT_XOR);
		Sprite16(count1*16, 0, 16, image_buffer_dark, dark, SPRT_XOR);
		
		//----------------------------------------------------------------
		// Starts on the 2nd row of pictures, and moves the pictures up.
		//----------------------------------------------------------------
		for (count2=1;count2<8;count2++)	//Row
		{
			Sprite16(count1*16,count2*16-screen_shift,16,gfx_light[ room_buffer[count2+screen_row_gain][count1][TERRAIN_PIC] ],light,SPRT_XOR);
			Sprite16(count1*16,count2*16-screen_shift,16,gfx_dark[ room_buffer[count2+screen_row_gain][count1][TERRAIN_PIC] ],dark,SPRT_XOR);
		}
	}
}


/*===========================================================================*/
/* Animates terrain on the screen by cycling through frames from             */
/* terrain_animation_list.                                                   */
/*===========================================================================*/
void AnimateTerrain(void)
{	
	int list_num = 0;	//Cycles through the terrain_animation_list
	
	//--------------------------------------------------
	// Defines how fast an animation moves. The larger 
	// ani_speed is, the slower it moves.
	//--------------------------------------------------
	int ani_speed = 0;
	
	//---------------------------------------------------
	// Each animated terrain cycles through 4 pictures.
	// ani_list holds the picture number for these pics
	//---------------------------------------------------
	int ani_list[] = {0,0,0,0};
	
	//-----------------------------------------------------
	// Scrolls through the animated terrains in the room
	//-----------------------------------------------------
	while (terrain_animation_list[list_num][TERRAIN_NUMBER] >= 0)
	{
		switch (terrain_animation_list[list_num][TERRAIN_NUMBER])
		{
			case 12: //FLower1
				ani_speed = 7;
				ani_list[0] = 12;
				ani_list[1] = 58;
				ani_list[2] = 60;
				ani_list[3] = 59;
				break;			
			case 20: //Water1
				ani_speed = 8;
				ani_list[0] = 20;
				ani_list[1] = 61;
				ani_list[2] = 62;
				ani_list[3] = 63;
				break;
			case 45: //Flying Rooster
				ani_speed = 3;
				ani_list[0] = 45;
				ani_list[1] = 64;
				ani_list[2] = 65;
				ani_list[3] = 66;
				break;
			case 86: //Water5 (the water at the beach)
				ani_speed = 7;
				ani_list[0] = 86;
				ani_list[1] = 87;
				ani_list[2] = 88;
				ani_list[3] = 89;
				break;
			case 85: //Shore1 (shore water at the beach)
				ani_speed = 7;
				ani_list[0] = 85;
				ani_list[1] = 90;
				ani_list[2] = 91;
				ani_list[3] = 92;
				break;
			//-----------------------------------------------------
			// Light - found inside houses. This routine is a
			// little different, so it overrides the default
			// routine of handling terrain animation.
			//-----------------------------------------------------
			case 124:
				switch (terrain_animation_list[list_num][TERRAIN_FRAME])
				{
					case 1 ... 4*1:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 124;
						break;
					case 4*1+1 ... 4*2:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 125;
						break;
					case 4*2+1 ... 4*3:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 126;
						break;
					case 4*3+1 ... 4*4+10:	//Short delay when the light is fully on
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 127;
						break;
					case 4*4+1+10 ... 4*5+10:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 126;
						break;
					case 4*5+1+10 ... 4*6+10:
						room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
						           [ terrain_animation_list[list_num][TERRAIN_Y] ]
						           [TERRAIN_PIC] = 125;
						break;
					default:
						terrain_animation_list[list_num][TERRAIN_FRAME] = 1;
					
				}
				terrain_animation_list[list_num][TERRAIN_FRAME]++;
				list_num++;
				continue;
		}
	
		//---------------------------------------------------------------------------
		//	Switches pictures, according to what frame the animated terrain is on
		//---------------------------------------------------------------------------		
		
		if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[0];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 2*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[1];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 2*ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 3*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[2];
		}
		else if (terrain_animation_list[list_num][TERRAIN_FRAME] >= 3*ani_speed+1 && terrain_animation_list[list_num][TERRAIN_FRAME] <= 4*ani_speed)
		{
			room_buffer[ terrain_animation_list[list_num][TERRAIN_X] ]
			           [ terrain_animation_list[list_num][TERRAIN_Y] ]
			           [TERRAIN_PIC] = ani_list[3];
		}
		//--------------------------------------------------
		//	Restarts the frame cycle if it reaches an end.
		//--------------------------------------------------
		else
		{
			terrain_animation_list[list_num][TERRAIN_FRAME] = 1;
		}
	
		//-----------------------------------------------------------------------
		//	Increase the frame number and list_num once everything is processed
		//-----------------------------------------------------------------------
		terrain_animation_list[list_num][TERRAIN_FRAME]++;
		
		list_num++;
		
	}
}


/*==================================================================================*/
/* This function checks if the given rectangular area collides with a non-walkable  */
/* terrain. If it does, it returns TRUE. Otherwise, it returns false.               */
/* NOTE: this function assumes the maximum area given will be 16x16.                */
/*==================================================================================*/
short CheckTerrainCollision(short x, short y, short width, short height)
{	
	//-----------------------------------------------------------------------
	// Any 16x16 area can potentially collide with 4 different terrains. So
	// 4 if-checks must be done, one on each of the terrains. The different
	// number additions/subtractions are for more travel leeway.
	//-----------------------------------------------------------------------
	if ((!(room_buffer[GetTerrainRow(y)][GetTerrainCol(x)][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, GetTerrainCol(x)*16+6, GetTerrainRow(y)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)][GetTerrainCol(x)+1][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, (GetTerrainCol(x)+1)*16+6, GetTerrainRow(y)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)+1][GetTerrainCol(x)][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, GetTerrainCol(x)*16+6, (GetTerrainRow(y)+1)*16+3, 16-6-6-1, 16-9-3)) ||
	    
	    (!(room_buffer[GetTerrainRow(y)+1][GetTerrainCol(x)+1][TERRAIN_PROPERTIES] & WALKABLE) &&
	     CheckAreaCollision(x, y, width, height, (GetTerrainCol(x)+1)*16+6, (GetTerrainRow(y)+1)*16+3, 16-6-6-1, 16-9-3)))
		return TRUE;
	else
		return FALSE;
}


/*======================================================================*/
/* Given the row and column of the terrain to slash, handles            */
/* whatever that terrain's slash does.                                  */
/*======================================================================*/
void SlashTerrain(short row, short col)
{
	switch (room_buffer[row][col][TERRAIN_PIC])
	{
		//--------------------------------------------------------
		// If Link slashes PLANT1 or GRASS1, change it to GRASS2
		//--------------------------------------------------------
		case 0:	//PLANT1
		case 1: //GRASS1
			room_buffer[row][col][TERRAIN_PROPERTIES] = WALKABLE;
			room_buffer[row][col][TERRAIN_PIC] = 2;	//GRASS2
			
			//---------------------------------------------------
			// Add the dissolving leaf animation to object_list
			//---------------------------------------------------
			AddObject(LEAF1, col*16, row*16);
			
			//---------------------------------------------------
			// Random chance of getting a heart or crystal
			//---------------------------------------------------
			if (!random(8))
			{
				//-----------------------------------
				// More crystals appear than hearts
				//-----------------------------------
				switch (random(3))
				{
					case 0:
						AddObject(HEART1, col*16, row*16);
						break;
					case 1 ... 2:
						AddObject(CRYSTAL1, col*16, row*16);
						break;
				}
			}
			break;
	}
}


/*======================================================================*/
/* Enters into an enterable terrain.                                    */
/*======================================================================*/
void EnterTerrain(short current_room, short row, short col)
{
	unsigned int count1;
	
	for (count1=0;count1<15;count1++)
	{
		OSContrastDn();
		DelayGame(30000);
	}
	
	switch (current_room)
	{
		//To Place where link starts off entrances
		case 1:
			link_map_col = 0;
			link_map_row = 0;
			link_x = 4*16;
			link_y = 7*16-10;
			LoadRoom(38);
			break;
		case 38:
			link_map_col = 2;
			link_map_row = 10;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(1);
			break;
		//Bowwow house entrances
		case 5:
			if (col == 3)
			{
				link_map_col = 1;
				link_map_row = 0;
				link_x=4*16;
				link_y = 7*16-10;
				LoadRoom(39);
			}
			else
			{
				link_map_col = 2;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(40);
			}
			break;
		case 39:
			link_map_col = 1;
			link_map_row = 10;
			link_x = 3*16;
			link_y = 3*16+8;
			LoadRoom(5);
			break;
		case 40:
			link_map_col = 1;
			link_map_row = 10;
			link_x = 5*16;
			link_y = 3*16+8;
			LoadRoom(5);
			break;
		//Phone roof house entrances
		case 4:
			link_map_col = 3;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(41);
			break;
		case 41:
			link_map_col = 2;
			link_map_row = 11;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(4);
			break;
		//Ulrira house entrances
		case 6:
			link_map_col = 4;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(42);
			break;
		case 42:
			link_map_col = 1;
			link_map_row = 11;
			link_x = 4*16;
			link_y = 5*16+8;
			LoadRoom(6);
			break;
		//Papahl's house entrances
		case 12:
			if (col == 5)
			{
				link_map_col = 5;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(43);
			}
			else
			{
				link_map_col = 6;
				link_map_row = 0;
				link_x=5*16;
				link_y = 7*16-10;
				LoadRoom(44);
			}
			break;
		case 43:
			link_map_col = 2;
			link_map_row = 8;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(12);
			break;
		case 44:
			link_map_col = 2;
			link_map_row = 8;
			link_x = 7*16;
			link_y = 4*16+8;
			LoadRoom(12);
			break;
		//Item shop entrances
		case 9:
			link_map_col = 7;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(45);
			break;
		case 45:
			link_map_col = 3;
			link_map_row = 9;
			link_x = 4*16;
			link_y = 5*16+8;
			LoadRoom(9);
			break;
		//Trendy game entrances
		case 3:
			link_map_col = 8;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(46);
			break;
		case 46:
			link_map_col = 3;
			link_map_row = 11;
			link_x = 5*16;
			link_y = 4*16+8;
			LoadRoom(3);
			break;
		//School entrances
		case 17:
			link_map_col = 9;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(47);
			break;
		case 47:
			link_map_col = 0;
			link_map_row = 11;
			link_x = 3*16;
			link_y = 2*16+8;
			LoadRoom(17);
			break;
		//Alligator house entrances
		case 29:
			link_map_col = 10;
			link_map_row = 0;
			link_x=5*16;
			link_y = 7*16-10;
			LoadRoom(48);
			break;
		case 48:
			link_map_col = 3;
			link_map_row = 14;
			link_x = 4*16;
			link_y = 2*16+8;
			LoadRoom(29);
			break;
	}
	
	DrawScreen();
	row++;
	
	for (count1=0;count1<9;count1++)
	{
		OSContrastUp();
		DelayGame(30000);
	}
}


/*=================================================*/
/* Calculates the terrain's row, given a y         */
/*=================================================*/
short GetTerrainRow(short y)
{
	return (int)(y/16);
}


/*=================================================*/
/* Calculates the terrain's column, given an x     */
/*=================================================*/
short GetTerrainCol(short x)
{
	return (int)(x/16);
}

















