// Header File
// Created 7/18/2011; 1:29:56 AM
/*---------------------------------------------------------------------------*/
/* Aliases                                                                   */
/*---------------------------------------------------------------------------*/
enum {UP, RIGHT, DOWN, LEFT};            //Easier reference to direction
enum {TERRAIN, MOB, LINK, OBJECT};       //General Types used in functions like DrawImage, and ShiftImage
enum {OBJECT_NUMBER, OBJECT_X, OBJECT_Y, OBJECT_FRAME}; //Used in object_list
enum {TERRAIN_NUMBER, TERRAIN_X, TERRAIN_Y, TERRAIN_FRAME}; //Used in terrain_animation_list
enum {TERRAIN_PIC, TERRAIN_PROPERTIES}; //References to terrain data (defined below)
enum {MOB_NUMBER, MOB_X, MOB_Y, MOB_FRAME, MOB_DIRECTION, MOB_PAUSE, MOB_VAR1, MOB_VAR2};  //Used in mob_list
enum {MOB_WANDER, MOB_FLUTTER, MOB_HOP, MOB_FOLLOW};  //Movement Types, used in function MoveMob

//Found on the beach
#define CRAB1       0	
#define CRAB2       1

//Usually found in town
#define BUTTERFLY1  2	
#define BUTTERFLY2  3

//Classic Zelda Octapus-looking monster
#define MONSTER1    4	
#define MONSTER2    5
#define MONSTER3    6
#define MONSTER4    7
#define MONSTER5    8
#define MONSTER6    9
#define MONSTER7    10
#define MONSTER8    11

//Found on the beach (spikey looking things)
#define URCHIN1     12	
#define URCHIN2     13
#define URCHIN3     14
#define URCHIN4     15

//Usually found in town
#define DOG1        16	
#define DOG2        17
#define DOG3        18
#define DOG4        19

//Usually found in town
#define CHICKEN1    20
#define CHICKEN2    21
#define CHICKEN3    22
#define CHICKEN4    23

//These are the creatures that come up from the ground,
//twirl around a bit, then disappears into the sand again.
#define SANDMOB1    24
#define SANDMOB2    25
#define SANDMOB3    26
#define SANDMOB4    27

//-------------------
// Bitwise booleans
//-------------------
#define WALKABLE    0x01
#define ANIMATED    0x02
#define ENTERABLE   0x04
#define FALLABLE    0x08
#define LIFTABLE    0x10
#define BOMBABLE    0x20

//--------------------------------------------------------------------------------------------------------------
// The following data is organized:
// #define NEW_TERRAIN {TERRAIN_PIC, TERRAIN_PROPERTIES}
// where:
// 1) TERRAIN_PIC is the picture number in img_map_light and img_map_dark, defined in "images.h"
// 2) TERRAIN_PROPERTIES is a byte, with each bit representing a boolean. These booleans are shown in the Alias section
//    of this header.
//
// To add another slot of data, you must also update: 1) TERRAIN value of room_list, 2) size of room_buffer, and
// 3) count3 in function LoadRoom()
//---------------------------------------------------------------------------------------------------------------

#define PLANT1    {0,0}
#define GRASS1    {1,WALKABLE}
#define GRASS2    {2,WALKABLE}
#define FENCE1    {3,0}
#define TREE1     {4,0}
#define TREE2     {5,0}
#define TREE3     {6,0}
#define TREE4     {7,0}
#define ROOF10    {8,0}
#define HOUSE2    {9,0}
#define HOUSE3    {10,WALKABLE | ENTERABLE}
#define ROAD1     {11,WALKABLE}
#define FLOWER1   {12,WALKABLE | ANIMATED}
#define EDGE1     {13,0}
#define EDGE2     {14,0}
#define EDGE3     {15,0}
#define EDGE4     {16,0}
#define EDGE5     {17,0}
#define HOUSE4    {18,WALKABLE | ENTERABLE}
#define HOUSE5    {19,0}
#define WATER1    {20,ANIMATED}
#define TREE5     {21,0}
#define ROOF1     {22,0}
#define ROOF2     {23,0}
#define ROOF3     {24,0}
#define ROOF4     {25,0}
#define ROOF5     {26,0}
#define ROOF6     {27,0}
#define ROAD2     {28,WALKABLE}
#define ROAD3     {29,WALKABLE}
#define FENCE2    {30,0}
#define ROOF7     {31,0}
#define ROOF8     {32,0}
#define ROOF9     {33,0}
#define TREE6     {34,0}
#define FENCE3    {35,0}
#define FENCE4    {36,0}
#define EDGE6     {37,0}
#define EDGE7     {38,0}
#define EDGE8     {39,0}
#define EDGE9     {40,0}
#define CLIFF1    {41,0}
#define CLIFF2    {42,0}
#define STAIR1    {43,WALKABLE}
#define CLIFF3    {44,0}
#define OBJECT1   {45,ANIMATED}
#define OBJECT2   {46,0}
#define ROCK1     {47,0}
#define EDGE10    {48,0}
#define EDGE11    {49,0}
#define EDGE12    {50,0}
#define EDGE13    {51,WALKABLE | FALLABLE}
#define EDGE14    {52,0}
#define EDGE15    {53,0}
#define SIGN1     {54,0}
#define EDGE16    {55,0}
#define CLIFF4    {56,0}
#define WELL1     {57,0}
#define FLOWER2   {58,WALKABLE | ANIMATED}
#define FLOWER3   {59,WALKABLE | ANIMATED}
#define FLOWER4   {60,WALKABLE | ANIMATED}
#define WATER2    {61,ANIMATED}
#define WATER3    {62,ANIMATED}
#define WATER4    {63,ANIMATED}
#define OBJECT3   {64,ANIMATED}
#define OBJECT4   {65,ANIMATED}
#define OBJECT5   {66,ANIMATED}
#define EDGE17    {67,0}
#define EDGE18    {68,WALKABLE | FALLABLE}
#define HOLE1     {69,0}
#define EDGE19    {70,0}
#define DUNG1     {71,0}
#define DUNG2     {72,0}
#define DUNG3     {73,0}
#define DUNG4     {74,0}
#define ROAD4     {75,WALKABLE}
#define SAND1     {76,WALKABLE}
#define PALM1     {77,0}
#define PALM2     {78,0}
#define PALM3     {79,0}
#define PALM4     {80,0}
#define SAND2     {81,WALKABLE}
#define SAND3     {82,WALKABLE}
#define SAND4     {83,0}
#define CHEST1    {84,0}
#define SHORE1    {85,WALKABLE | ANIMATED}
#define WATER5    {86,ANIMATED}
#define WATER6    {87,ANIMATED}
#define WATER7    {88,ANIMATED}
#define WATER8    {89,ANIMATED}
#define SHORE2    {90,WALKABLE | ANIMATED}
#define SHORE3    {91,WALKABLE | ANIMATED}
#define SHORE4    {92,WALKABLE | ANIMATED}
#define SAND5     {93,WALKABLE}
#define CLIFF5    {94,0}
#define CLIFF6    {95,0}
#define EDGE20    {96,0}
#define WALL1     {97,0}
#define WALL2     {98,0}
#define WALL3     {99,0}
#define WALL4     {100,0}
#define WALL5     {101,0}
#define WALL6     {102,0}
#define WALL7     {103,0}
#define WALL8     {104,0}
#define TABLE1    {105,0}
#define TABLE2    {106,0}
#define POT1      {107,0}
#define FLOOR1    {108,WALKABLE}
#define FLOOR2    {109,WALKABLE}
#define CHAIR1    {110,0}
#define BOX1      {111,0}
#define BED1      {112,0}
#define BED2      {113,0}
#define CABINET1  {114,0}
#define CABINET2  {115,WALKABLE}
#define DOOR2     {116,WALKABLE | ENTERABLE}
#define TABLE3    {117,0}
#define BLACK1    {118,0}
#define SAND6     {119,WALKABLE}
#define PHONE1    {120,WALKABLE}
#define PHONE2    {121,0}
#define PHONE3    {122,WALKABLE}
#define PHONE4    {123,0}
#define LIGHT1    {124,ANIMATED}
#define LIGHT2    {125,ANIMATED}
#define LIGHT3    {126,ANIMATED}
#define LIGHT4    {127,ANIMATED}
#define CASHIER1  {128,0}
#define FLOOR3    {129,0}
#define WHITE1    {130,0}
#define TABLE4    {131,0}
#define SHELF1    {132,0}
#define SHELF2    {133,WALKABLE}

// 31 walk, 24 animated, 8 walk/animated, 78 other  ... 40 walk. 40 animated.  20 walking animated?
// 100 other.  you also need a set for hookshot grabable.  maybe from 175->199? 


//Shadows found under mobs when jumping
#define SHADOW1    0

//Link's sword, representing 7 swing directions
#define SWORD1     1
#define SWORD2     2
#define SWORD3     3
#define SWORD4     4
#define SWORD5     5
#define SWORD6     6
#define SWORD7     7

//Displayed when Link chops a bush
#define LEAF1      8
#define LEAF2      9
#define LEAF3      10
#define LEAF4      11

//Displayed when enemies get killed
#define EXPLODE1   12
#define EXPLODE2   13
#define EXPLODE3   14

//After chopping bushes or enemies you can get hearts and crystals
#define HEART1     15
#define CRYSTAL1   16
//-----------------------------
// Aliases pertaining to Link
//-----------------------------

//State when he is just standing
#define LINK_NORMAL   0

//State when player pushes the movement keys
#define LINK_WALK     1

//State when Link thrusts his sword
#define LINK_THRUST   2
#define LINK_THRUST2  3

//State when Link is jumping or falling
#define LINK_JUMP     4
#define LINK_JUMP2    5
#define LINK_JUMP3    6


/*---------------------------------------------------------------------------*/
/* Global Variables                                                          */
/*---------------------------------------------------------------------------*/

char *light = 0;                         //Pointer to virtual light plane
char *dark = 0;                          //Pointer to virtual dark plane

INT_HANDLER save_int_1;                  //Save interrupt 1 (replaced with DUMMY_HANDLER)

short exited = FALSE;                    //TRUE if user exited, FALSE otherwise

unsigned short image_buffer_light[16];	 //Used to store temporarily modified pictures
unsigned short image_buffer_dark[16];    //Used to store temporarily modified pictures
unsigned short image_buffer_mask[16];    //Used to store temporarily modified pictures

short screen_shift = 0;                  //screen shift is explained in function DrawTerrain() from "terrain.h"
short screen_row_gain = 0;               //screen_row_gain is explained in function DrawTerrain() from "terrain.h"

short key_lock_A = FALSE;                //TRUE if the player is holding down 2nd, FALSE otherwise
short key_lock_B = FALSE;                //TRUE if the player is holding down Shift (the key to the right of 2nd), FALSE otherwise

int room_buffer[8][10][2];               //Used to store current room (so original wont be modified)
short terrain_animation_list[80][4];     //List of terrain animations to be drawn
short mob_list[5][8];                    //List used to store current mobs in the room
short object_list[50][4];                //List used to store current objects in the room


//--------------------------------------
// Global Variables pertaining to Link
//--------------------------------------
short link_x = 5*16;                       //Link's x position on the screen
short link_y = 5*16;                        //Link's y position on the screen

short link_map_col = 2;                    //Link's column room position in the world, starts at 2
short link_map_row = 10;                   //Link's row room position in the world, starts at 10

short link_speed = 3;                    //How fast link is walking (in pixels)
short link_direction = DOWN;               //Direction link is facing

short link_picture = LINK_NORMAL;        //Stores current picture of Link.
short link_state = LINK_NORMAL;          //Link's current state, for example: walk, normal, jump, etc.

int   link_frame = 0;                    //Keeps track of link's frame #.