// C Source File
// Created 7/17/2011; 1:35:58 AM
/*******************************************************************************/
/* Zelda Crystal Engine                                                       */
/* -------------------------------------------                                 */
/* By: Kyle ingalsbe                                                           */
/*                                                                             */
/* Email: kyle_ingalsbe@yahoo.com                                              */
/*                                                                             */
/*******************************************************************************/
#include <tigcclib.h>
#include "all.h"


extern int link_frame, button_pressed,link_frame2, link_x, link_y;


//=================================================================================================
/*
			compile_mobs() will look at the room stats to determine if there exists mobs in it.  if a 
			mob file is specified then it will load the file from archive and then use it to populate
			the mob_list[][] in the dll.  however, if the room specifies that it is to randomly choose
			mobs, then compile_mobs() will pull the random_mob_generator() and exit.  compile_mobs()
			can also obtain a mob list file from the world[][] stats. 
			
			the variable "which" has 2 settings, which will tell it what mob_list to store the info
			into, in the dll:
			
			1 - store to mob_list[][]
			2 - store to world_mob_list[][]

			file= -88 is to let you know that you need to randomly generate a certain amt of mobs.
			in that room. number_of_mobs is used to tell how many mobs to generate.			
*/
//=================================================================================================


void compile_mobs(int file, int start, int end, int which, int number_of_mobs)
{
	//------------------------------------------------------------------
	//  fix out of bounds errors
	//------------------------------------------------------------------

	if (start>=max_mobs){start=max_mobs-1;}
	if (end>=max_mobs){end=max_mobs-1;}
	if (number_of_mobs>=max_mobs){number_of_mobs=max_mobs;}
	
	//------------------------------------------------------------------
	//  if -88, then pull the random mob generator, also if <0, then
	//  quit out to prevent errors
	//------------------------------------------------------------------
	
	if (file<=0){if (file==(-88)){random_mob_generator(start,end,number_of_mobs);}return;}

	
	int tmp[max_mobs][mob_num_of_stats];
	int x=start,y=0,i=0,k=0;

	if (which == 2) {start=0;end=max_mobs-1;}
	
 TRY
   FILE *f = NULL;
    
		char sav[8];
    sprintf(sav,"zmbs%i",file);			
    FolderCur(SYMSTR("mobs"),TRUE);

		//------------------------------------------------------------------
		//  pull the file and load it into the dll
		//------------------------------------------------------------------

    if ((f = fopen(sav,"rb")) == NULL) {} else {
       if (fread(tmp,sizeof(tmp),1,f) != 1) {}
        else {
        		 
								//------------------------------------------------------------------
								//  if we do a range with a certain number of mobs, then cycle through
								//  the range of mobs specified until you reached the number of mobs
								//  specified.
								//------------------------------------------------------------------							
								if ((which==1) && (number_of_mobs>(end-start))){
									
								
									for (i=0;i<number_of_mobs;i++)
									{
											for (y=0;y<mob_num_of_stats;y++){s_mob_list(i,y,tmp[x][y]);}// end for set
											if ((get_mob_stat(i,mob_x)<=0) && (get_mob_stat(i,mob_y)<=0)){random_mob_placer(i);}
											x++;
											if (x>end){x=start;}	
									}// end for
											while (i<max_mobs){while (k<mob_num_of_stats){s_mob_list(i,k,(-100));	k++;}i++;k=0;}// end i<max mobs
										return;
								}// end if
								else {
								
				       		 for (x=0;x< max_mobs; x++){
				        		 		for (y=0;y<mob_num_of_stats;y++){
				        		 			if (which==1){s_mob_list(x,y,tmp[x][y]);}	
				        		 			else if (which==2){s_world_mob_list(x,y,tmp[x][y]);}	
				        				}//end for 2
				        		}// end for 1		
        				}// end else
         } //  yn = FALSE;

        fclose(f);
    }

	ONERR
		return;// prevents addr errors
	ENDTRY

	k=0;
	i=0;
	//------------------------------------------------------------------
	//  now run through the list.  If in range then only clear out
	//  the mob_frame.  however, if not in range, then delete out the
	//  rest of the unused mobs.  Also, run a check to see if the mobs
	//  are not placed, meaning that they have an x/y location of <=0.
	//  If they do, then randomly place them somewhere.
	//------------------------------------------------------------------
	
	while (i<max_mobs){
		
		if (i>=start && i<=end){if ((get_mob_stat(i,mob_x)<=0) && (get_mob_stat(i,mob_y)<=0)){random_mob_placer(i);}}
		else {
				while (k<mob_num_of_stats)
				{
					if (which==1){s_mob_list(i,k,(-100));}
					else if (which==2){s_world_mob_list(i,k,(-100));}
					k++;
				}// end while k<15
		}//end else if not between beginning and end.
		if (which==1){s_mob_list(i,mob_frame,0);}
		else if (which==2){s_world_mob_list(i,mob_frame,0);}
		i++;k=0;
	}// end while i<64
	
}// end compile mobs


//=================================================================================================
/*
			slashmob() will loop through the mob list and test to see if link's sword hit a mob.
*/
//=================================================================================================

int SlashMob(int x, int y, int width, int height)
{
	int count1, yn=FALSE, stat=0;
	//------------------------------------------------------------------
	// loop through the entire mob list to test every mob to see if
	// it has been hit.
	//------------------------------------------------------------------
	
	for (count1=0;count1<max_mobs;count1++)
	{


		//------------------------------------------------------------------
		//  check to see if the sword collides with the mob and that the 
		//  mob is not underground.  (you cannot hit a mob underground).
		//------------------------------------------------------------------


		if ( CheckAreaCollision(x,y,width,height, get_mob_stat(count1,mob_x), get_mob_stat(count1,mob_y), 
		get_mob_stat(count1,mob_w), get_mob_stat(count1,mob_h)) &&  get_mob_stat(count1,mob_id) > 0  
		&& get_mob_stat(count1,mob_underground)<1 )
		{
		
			//------------------------------------------------------------------
			//  if the mob is holding a shield and it is facing link, then 
			//  deal no damage to it.  otherwise, deal damage.
			//------------------------------------------------------------------
			stat=get_mob_stat(count1, mob_shield);
			if (((stat==shield_equipped) || (stat==nostun_shield_equipped) || (stat==nostun_shield_immune_magic)) 
					&& (link_facing(count1)==TRUE)){}
			else if (get_mob_stat(count1, mob_hit)<=0){mod_mob(count1,mob_hit,5,TRUE);}// end else
			
			yn=TRUE;// yes the mob is hit
			
			//------------------------------------------------------------------
			//  after link hits the mob, return him to normal
			//------------------------------------------------------------------
			if (((link_stat(dual_state)==LINK_THRUST)&&(link_frame>=2)) || (link_stat(dual_state)==LINK_WALKING_SWORD))
			{	mod_link(dual_state,0, TRUE);	link_frame = 0;}
	
		}// end if you did hit the mob.
		
	}// end for loop
	return yn;
}// end slash mob


//=================================================================================================
/*
			random_mob_placer() randomly picks 2 numbers and tests the sprite if it is walkable.  
			if so, then the mob	is placed there.  it is used in conjunction with the random mob generator
			and with the "teleporting" of the wizard mobs.
*/
//=================================================================================================


void random_mob_placer(int who)
{
			int cx=0,cy=0,trys=0;
			
	redo:
			//------------------------------------------------------------------
			//  randomly choose a sprite:
			//------------------------------------------------------------------

       cx=random(10)+1;
       if (cx>8){cx=8;}if (cx<1){cx=1;}
       cy=random(8)+1;
       if (cy>6){cy=6;}if (cy<1){cy=1;}

				//------------------------------------------------------------------
				//  if walkable sprites, then place mob
				//  and if not within 27 pixles of link, as well - got tired of
				//  walking into a room and having a mob stand where i was.
				//------------------------------------------------------------------

				if ((((get_all_maps(rmapx, rmapy,cy,cx)>=0) && (get_all_maps(rmapx, rmapy,cy,cx)<=49)) ||
					 ((get_all_maps(rmapx, rmapy,cy,cx)>=66) && (get_all_maps(rmapx, rmapy,cy,cx)<=78)))  && 
					 (!((link_x-27<cx*16) && (link_x+27>cx*16) && (link_y-27<cy*16) && (link_y+27>cy*16))))
				{
						
						mod_mob(who, mob_x, cx*16, TRUE);
       			 mod_mob(who, mob_y, cy*16, TRUE);
				}// end if a place a mob can stand
				
				//------------------------------------------------------------------
				//  if not walkable, then try again, but no more than 300 times.
				//------------------------------------------------------------------
				else
				{
					trys++;
					if (trys>300){goto getout;}
					goto redo;
				}
	getout:
}//end mob placer


//=================================================================================================
/*
		random_mob_generator() checks to see if the room stat (mob file) == -88.  if it does, then 
		it means to randomly choose mobs from the world[][] mob file.  This will clear the room of
		mobs, then pull the gfx file for the world[][] mob list.  then it will choose a	number of 
		mobs (specified by the room stat (num of mobs)) from the range specified by room stats: 
		(mob start and mob end).  It will compile a list of mobs for the current room. 
*/
//=================================================================================================


void random_mob_generator(int start, int end, int number_of_mobs)
{

	//------------------------------------------------------------------
	//  clear out the mobs in the room
	//------------------------------------------------------------------

	clr_mob_list(1);//clr_mob_list(2); - dont do that.  it makes all mobs dissappear
		
		mob_gfx_loader(g_over_world_stat(overworld_mob_gfx_file, oob()));// pull the gfx file just in case.

		int i=0,k=0,num=0;

		//------------------------------------------------------------------
		//  check out of bounds info
		//------------------------------------------------------------------

		if (number_of_mobs<=0){goto end;}
		if (start>=max_mobs){start=max_mobs-1;}

		//-------------------------------------------------------------------
		// if a user juxtaposes the start and end mob range, then reverse it
		//-------------------------------------------------------------------

		if (end<start) {end=start+(start-end);}
		

		if (end>=max_mobs){end=max_mobs-1;}
		if (number_of_mobs>max_mobs){number_of_mobs=max_mobs;}
		if (start<0){start=0;}


		//------------------------------------------------------------------
		// randomly choose the mobs based upon the range and place it into
		// the mob_list[][] in the dll
		//------------------------------------------------------------------
				
		randomize();

			while (i<number_of_mobs)
			{
				num=random((end+1)-start)+(start);

				if (num<start){num=start;}
				if (num>end){num=end;}

				for (k=0;k<mob_num_of_stats;k++){s_mob_list(i,k,g_world_mob_list(num,k));}

					//------------------------------------------------------------------
					//  after loading the mob into the dll, find a place for it to 
					//  stand on the map
					//------------------------------------------------------------------

					random_mob_placer(i);// now find a place for it to stand

				i++;
			}// end while
		
		//------------------------------------------------------------------
		//  if you have completed getting all the mobs specified by the
		//  room file, then zero out any left over spaces in the mob_list[][]
		//  so we dont have any phantom mobs lurking around the room.
		//------------------------------------------------------------------

		while (i<max_mobs)
		{
				while (k<mob_num_of_stats)
				{
					s_mob_list(i,k,(-100));
					k++;
				}
				i++;k=0;
		}// end i<max mobs

	end:
}// end compile mobs




//=================================================================================================

int find_mob_spot()
{
	int i=0;
	for (i=0;i<max_mobs;i++){	if (g_mob_list(i,0)<0){return i;}	}// end i<max mobs
	return 555;
}


//=================================================================================================

int add_a_mob(int xloc, int yloc, int loc, int file)
{
	int i=0, y=0, yn=FALSE;
	if ((i=find_mob_spot())!=555)
	{
	
TRY
   FILE *f = NULL;
    int tmp[max_mobs][mob_num_of_stats];
		char sav[8];
    sprintf(sav,"zmbs%i",file);			
    FolderCur(SYMSTR("mobs"),TRUE);

		//------------------------------------------------------------------
		//  pull the file and load it into the dll
		//------------------------------------------------------------------

    if ((f = fopen(sav,"rb")) == NULL) {} else {
       if (fread(tmp,sizeof(tmp),1,f) != 1) {}
        else {
        				for (y=0;y<mob_num_of_stats;y++){s_mob_list(i,y,tmp[loc][y]);}// end for set
								if ((xloc<=0) && (yloc<=0)){random_mob_placer(i);}
								else {
									mod_mob(i, mob_x, xloc, TRUE);
									mod_mob(i, mob_y, yloc, TRUE);
								}// end else (xloc<=0) && (yloc<=0))
							yn=TRUE;		
	       } //  yn = FALSE;

        fclose(f);
    }

	ONERR
		return yn;// prevents addr errors
	ENDTRY
	}// end if 
	return yn;
}

//=================================================================================================

void pain(int who, int dam, int dam_type, int damager)
{
	pain2(who, dam, dam_type, damager);
	if ((who!=LINK) && (get_exec_sl()>0))
		{
			get_story(get_exec_sl());// pull the storyline file	
			story_main();// execute the storyline file.
		}
	clr_exec_sl();
}


//=================================================================================================

